@@ -1,8 +1,11 @@
 /* alarm.c -- set the alarm power on time of a Toshiba laptop.
  *
- * Copyright (c) 1998-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1998-2001  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: alarm.c,v $
+ * Revision 1.3  2001/02/02 15:38:35  jab
+ * added notice about kernel module to error message
+ *
  * Revision 1.2  2000/02/10 23:49:17  jab
  * fixed reporting of the year the alarm dates are set to
  *
@@ -26,7 +29,7 @@
  *
  */
 
-static char const rcsid[]="$Id: alarm.c,v 1.2 2000/02/10 23:49:17 jab Exp jab $";
+static char const rcsid[]="$Id: alarm.c,v 1.3 2001/02/02 15:38:35 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -42,8 +45,8 @@ static char const rcsid[]="$Id: alarm.c,v 1.2 2000/02/10 23:49:17 jab Exp jab $"
 
 
 #define VERSION_STRING "\
-Alarm version 1.1\n\
-Copyright (c) 1998-2000 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
+Alarm version 1.2\n\
+Copyright (c) 1998-2001 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
 
 #define USAGE_STRING "\
 Usage: alarm [-d] [-l] [-h] [-v] time\n\
@@ -127,7 +130,8 @@ int main(int argc, char *argv[])
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
-		fprintf(stderr, "alarm: this computer is not supported\n");
+		fprintf(stderr, "alarm: this computer is not supported "
+			"or the kernel module is not installed.\n");
 		return 1;
 	}
 
