@@ -1,8 +1,13 @@
 /* fan.c -- turn the internal fan on/off in a Toshiba Pentium(tm) laptop.
  *
- * Copyright (c) 1996,97,98  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1996,97,98,2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: fan.c,v $
+ * Revision 3.5  2001/01/03 16:11:08  jab
+ * added notice about kernel module to error message
+ * moved the lock file for the daemon option to /var/tmp
+ * now uses the kernel module interface to the HCI
+ *
  * Revision 3.4  1999/12/01 19:59:23  jab
  * added daemon option to keep fan turned on when on external power
  * sorted out the assembler so it compiles and works whatever gcc is used
@@ -65,7 +70,7 @@
  *
  */
 
-static char const rcsid[]="$Id: fan.c,v 3.4 1999/12/01 19:59:23 jab Exp jab $";
+static char const rcsid[]="$Id: fan.c,v 3.5 2001/01/03 16:11:08 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -86,7 +91,7 @@ static char const rcsid[]="$Id: fan.c,v 3.4 1999/12/01 19:59:23 jab Exp jab $";
 
 enum { ON, OFF, STATUS, AUTO, TOGGLE };
 
-#define PID_FILE _PATH_VARRUN "fan.pid"
+#define PID_FILE _PATH_VARTMP ".fan.pid"
 
 
 void FanOn(char *name)
@@ -248,7 +253,8 @@ int main(int argc, char *argv[])
 	/* check to make sure laptop has a fan */
 
 	if (FanStatus()<0x00) {
-		fprintf(stderr, "fan: laptop does not have cooling fan.\n");
+		fprintf(stderr, "fan: laptop does not have cooling fan "
+			"or kernel module not installed.\n");
 		return 1;
 	}
 
