@@ -1,8 +1,16 @@
 /* hotkey.c -- deal with Fn+<KEY> presses on a Toshiba laptop under Linux
  *
- * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1997-2001  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: hotkey.c,v $
+ * Revision 2.5  2001/02/02 17:09:00  jab
+ * added notice about kernel module to error message
+ * moved the lock file to /var/tmp
+ *
+ * Revision 2.4  2000/06/23 22:18:48  jab
+ * patch from Dennis Josifovich <dennis_josifovich@somerset-financial.com> to
+ * recognize the -display command line option
+ *
  * Revision 2.3  2000/02/12 21:35:30  jab
  * fix so Stockholm class models display economy power graphic
  *
@@ -60,7 +68,7 @@
  *
  */
 
-static char const rcsid[]="$Id: hotkey.c,v 2.3 2000/02/12 21:35:30 jab Exp jab $";
+static char const rcsid[]="$Id: hotkey.c,v 2.5 2001/02/02 17:09:00 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -106,7 +114,7 @@ static int depth = 0;
 #define FONT "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*,\
 -*-*-*-r-normal--14-*-*-*-*-*"
 
-#define PID_FILE _PATH_VARRUN "hotkey.pid"
+#define PID_FILE _PATH_VARTMP ".hotkey.pid"
 
 #define USAGE \
 "Usage: hotkey [OPTION]\n\n\
@@ -472,12 +480,14 @@ int main(int argc, char *argv[])
 	XEvent event;
 	XColor color;
 	char **list,*def_string,*locale,buffer[64];
+	char *monitor = NULL;
 
 
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
-		fprintf(stderr, "hotkey: this computer is not supported\n");
+		fprintf(stderr, "hotkey: this computer is not supported "
+			"or the kernel module is not installed.\n");
 		return 1;
 	}
 
@@ -536,6 +546,16 @@ int main(int argc, char *argv[])
 
 	if (--argc>0) {
 		argv++;
+		if (strcmp(argv[0], "-display")) {
+			if (argc<=1) {
+				printf(USAGE);
+				unlink(PID_FILE);
+				return -1;
+			}
+			monitor = argv[1];
+			argc--;
+			argv++;
+		}
 		if (!strcmp(argv[0], "-h")) {
 			printf(USAGE);
 			unlink(PID_FILE);
@@ -590,9 +610,15 @@ int main(int argc, char *argv[])
 		return 1;
 	}
  
-	/* try and open the local display */
+	/* try and open the display */
 
-	if(!(display = XOpenDisplay(":0.0"))) {
+	if (monitor==NULL) {
+		monitor = getenv("DISPLAY");
+		if (monitor==NULL) {
+			monitor = ":0.0";
+		}
+	}
+	if(!(display = XOpenDisplay(monitor))) {
 		fprintf(stderr, "hotkey: unable to open display\n");
 		unlink(PID_FILE);
 		return 1;
