@@ -3,6 +3,9 @@
  * Copyright (c) 1996-98  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: machine.c,v $
+ * Revision 1.4  2001/02/04 22:40:20  jab
+ * fixed the option menu bug on the battery options page
+ *
  * Revision 1.3  2000/02/11 00:00:23  jab
  * fixed ambiguously nested if-elses
  * super bright LCD settings fixed courtesy of Tom May <tom@you-bastards.com>
@@ -30,7 +33,7 @@
  *
  */
 
-static const char rcsid[]="$Id: machine.c,v 1.3 2000/02/11 00:00:23 jab Exp jab $";
+static const char rcsid[]="$Id: machine.c,v 1.4 2001/02/04 22:40:20 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -362,6 +365,36 @@ void ResumeDependency(void)
 }
 
 
+/*
+ * Build an option menu
+ */
+GtkWidget* BuildMenu(MenuItem *list)
+{
+	GtkWidget *menu,*item;
+
+	menu = gtk_menu_new();
+	
+	while (list->label) {
+		item = gtk_menu_item_new_with_label(list->label);
+		gtk_menu_append(GTK_MENU(menu), item);
+
+		if (list->callback) {
+			gtk_signal_connect(GTK_OBJECT(item), "activate",
+				GTK_SIGNAL_FUNC(list->callback), list->data);
+		}
+
+		/* a bug in Gtk keeps gtk_widget_show_all(window) from handling
+		   these labels properly, so show them manually for now */
+
+		gtk_widget_show(item);
+		list->widget = item;
+		list++;
+	}
+
+	return menu;
+}
+
+
 /*
  * Get the number of the menu item corresponding to the setting else return -1
  */
@@ -378,6 +411,34 @@ int GetMenuItemNumber(MenuItem *menu, int setting)
 }
 
 
+/*
+ * Set sensitivity of the battery option menus depending on battery save mode
+ */
+void SetBatterySensitivity(int save)
+{
+
+	if (save==SCI_USER_SETTINGS) {
+		gtk_widget_set_sensitive(GTK_WIDGET(hdd), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(display), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(autooff), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(speed), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(sleep), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(lcd), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(cooling), TRUE);
+	} else {
+		gtk_widget_set_sensitive(GTK_WIDGET(hdd), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(display), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(autooff), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(speed), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(sleep), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(lcd), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
+	}
+
+	return;
+}
+
+
 /*
  * Set the battery option option menus to reflect the stored settings
  */
@@ -387,6 +448,10 @@ void SetBatteryOptions(int mode)
 	struct BatterySettings settings;
 
 
+	/* make the option menus sensitive to avoid some remaining greyed */
+
+	SetBatterySensitivity(SCI_USER_SETTINGS);
+
 	if (mode==SCI_MAINS) {
 		settings.save = external.save;
 		settings.hdd = external.hdd;
@@ -453,202 +518,15 @@ void SetBatteryOptions(int mode)
 	if ((i = GetMenuItemNumber(coolingmenu, settings.cooling))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(cooling), i);
 
-	return;
-}
 
+	/* we are done so set the enable/disable the option menus as needed */
 
-/*
- * Set sensitivity of the battery option menus depending on battery save mode
- */
-void SetBatterySensitivity(int save)
-{
-
-	if (save==SCI_USER_SETTINGS) {
-		gtk_widget_set_sensitive(GTK_WIDGET(hdd), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(display), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(autooff), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(speed), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(sleep), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(lcd), TRUE);
-		gtk_widget_set_sensitive(GTK_WIDGET(cooling), TRUE);
-	} else {
-		gtk_widget_set_sensitive(GTK_WIDGET(hdd), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(display), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(autooff), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(speed), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(sleep), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(lcd), FALSE);
-		gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
-	}
+	SetBatterySensitivity(settings.save);
 
 	return;
 }
 
 
-/*
- * Change the option menus when the power source is changed by the radio buttons
- */
-void PowerSourceCallback(GtkWidget *widget, int data)
-{
-	if (source!=(int) data) {
-
-	        source = (int) data;
-
-		if (source==SCI_MAINS)
-			SetBatterySensitivity(external.save);
-		else
-			SetBatterySensitivity(internal.save);
-
-		SetBatteryOptions(source);
-	}
-
-	return;
-}
-
-
-/*
- * Set the option menus to reflect the currently selected battery save option
- */
-void BatteryCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.save = data;
-	else
-		internal.save = data;
-
-	SetBatterySensitivity(data);
-	SetBatteryOptions(source);
-
-	return;
-}
-
-
-void HDDCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.hdd = data;
-	else
-		internal.hdd = data;
-
-	return;
-}
-
-
-void DisplayCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.display = data;
-	else
-		internal.display = data;
-
-	return;
-}
-
-
-void AutoOffCallback(GtkWidget *widget, int data)
-{
-	SMMRegisters reg;
-
-	reg.ebx = SCI_POWER_UP;
- 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		if ((reg.ecx & 0xffff)==SCI_BOOT)
-			ResumeDependency();
-	}
-
-	if (source==SCI_MAINS)
-		external.autooff = data;
-	else
-		internal.autooff = data;
-
-	return;
-}
-
-
-void SpeedCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.speed = data;
-	else
-		internal.speed = data;
-
-	return;
-}
-
-
-void SleepCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.sleep = data;
-	else
-		internal.sleep = data;
-
-	return;
-}
-
-
-void LCDCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.lcd = data;
-	else
-		internal.lcd = data;
-
-	return;
-}
-
-
-void CoolingCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.cooling = data;
-	else
-		internal.cooling = data;
-
-	return;
-}
-
-
-void StandbyCallback(GtkWidget *widget, int data)
-{
-	if (source==SCI_MAINS)
-		external.standby = data;
-	else
-		internal.standby = data;
-
-	return;
-}
-
-
-/*
- * Build an option menu
- */
-GtkWidget* BuildMenu(MenuItem *list)
-{
-	GtkWidget *menu,*item;
-
-	menu = gtk_menu_new();
-	
-	while (list->label) {
-		item = gtk_menu_item_new_with_label(list->label);
-		gtk_container_add(GTK_CONTAINER(menu), item);
-
-		if (list->callback) {
-			gtk_signal_connect(GTK_OBJECT(item), "activate",
-				GTK_SIGNAL_FUNC(list->callback), list->data);
-		}
-
-		/* a bug in Gtk keeps gtk_widget_show_all(window) from handling
-		   these labels properly, so show them manually for now */
-
-		gtk_widget_show(item);
-		list->widget = item;
-		list++;
-	}
-
-	return menu;
-}
-
-
 /*
  * Update the displayed sound settings so they reflect their actual settings
  */
@@ -756,6 +634,132 @@ void SetSystemOptions(void)
 }
 
 
+/*
+ * Change the option menus when the power source is changed by the radio buttons
+ */
+void PowerSourceCallback(GtkWidget *widget, int data)
+{
+	if (source!=(int) data) {
+	        source = (int) data;
+		SetBatteryOptions(source);
+	}
+
+	return;
+}
+
+
+/*
+ * Set the option menus to reflect the currently selected battery save option
+ */
+void BatteryCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.save = data;
+	else
+		internal.save = data;
+
+	SetBatteryOptions(source);
+
+	return;
+}
+
+
+void HDDCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.hdd = data;
+	else
+		internal.hdd = data;
+
+	return;
+}
+
+
+void DisplayCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.display = data;
+	else
+		internal.display = data;
+
+	return;
+}
+
+
+void AutoOffCallback(GtkWidget *widget, int data)
+{
+	SMMRegisters reg;
+
+	reg.ebx = SCI_POWER_UP;
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_BOOT)
+			ResumeDependency();
+	}
+
+	if (source==SCI_MAINS)
+		external.autooff = data;
+	else
+		internal.autooff = data;
+
+	return;
+}
+
+
+void SpeedCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.speed = data;
+	else
+		internal.speed = data;
+
+	return;
+}
+
+
+void SleepCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.sleep = data;
+	else
+		internal.sleep = data;
+
+	return;
+}
+
+
+void LCDCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.lcd = data;
+	else
+		internal.lcd = data;
+
+	return;
+}
+
+
+void CoolingCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.cooling = data;
+	else
+		internal.cooling = data;
+
+	return;
+}
+
+
+void StandbyCallback(GtkWidget *widget, int data)
+{
+	if (source==SCI_MAINS)
+		external.standby = data;
+	else
+		internal.standby = data;
+
+	return;
+}
+
+
 /*
  * Save the selected settings to the configuration file, and set them
  * appropriately for the current power source when closing the Machine
@@ -977,7 +981,7 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 	   need info on machine ID's for Atlanta and Derby machines */
 
 	reg.ebx = SCI_BATTERY_SAVE;
-	GetSetting(&reg, ERROR_GUI);
+	GetSetting(&reg, ERROR_NONE);
 	if ((reg.edx & 0xffff)==3)
 		savetype = STOCKHOLM;
 	else
@@ -985,7 +989,7 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 
 	lcdtype = BACKLIGHT;
 	reg.ebx = SCI_LCD_BRIGHTNESS;
-	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+	if (GetSetting(&reg, ERROR_NONE)==SCI_SUCCESS) {
 		if ((reg.ebx & 0xffff)==0x8001)
 			lcdtype = SUPER;
 		if ((reg.ebx & 0xffff)==0x0000)
@@ -1137,8 +1141,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(hdd), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(hdd), FALSE);
 
 	menu = BuildMenu(displaymenu);
 	display = gtk_option_menu_new();
@@ -1147,8 +1149,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(display), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(display), FALSE);
 
 	menu = BuildMenu(autooffmenu);
 	autooff = gtk_option_menu_new();
@@ -1157,8 +1157,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(autooff), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(autooff), FALSE);
 
 	menu = BuildMenu(speedmenu);
 	speed = gtk_option_menu_new();
@@ -1167,8 +1165,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(speed), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(speed), FALSE);
 
 	menu = BuildMenu(sleepmenu);
 	sleep = gtk_option_menu_new();
@@ -1177,8 +1173,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(sleep), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(sleep), FALSE);
 
 	if (lcdtype==BRIGHT)
 		menu = BuildMenu(lcdbrightmenu);
@@ -1193,8 +1187,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(lcd), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(lcd), FALSE);
 
 	menu = BuildMenu(coolingmenu);
 	cooling = gtk_option_menu_new();
@@ -1203,8 +1195,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(cooling), menu);
 	placement++;
-	if (powermode!=SCI_USER_SETTINGS)
-		gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
 
 	/* set the option menus to match the current battery save mode */
 
