@@ -1,23 +1,36 @@
 /* toshiba.c -- Linux driver for accessing the SMM on Toshiba laptops
  *
- * Copyright (c) 1996-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1996-2001  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * Valuable assistance and patches from:
  *     Tom May <tom@you-bastards.com>
  *     Rob Napier <rnapier@employees.org>
  *
- * Fn status port numbers for machine ID's coutesy of
+ * Fn status port numbers for machine ID's courtesy of
+ *     0xfc04: Steve VanDevender <stevev@efn.org>
+ *     0xfc08: Garth Berry <garth@itsbruce.net>
+ *     0xfc0a: Egbert Eich <eich@xfree86.org>
+ *     0xfc10: Andrew Lofthouse <Andrew.Lofthouse@robins.af.mil>
+ *     0xfc11: Spencer Olson <solson@novell.com>
+ *     0xfc13: Claudius Frankewitz <kryp@gmx.de>
  *     0xfc15: Tom May <tom@you-bastards.com>
- *     0xfc17: Mr. Dave Konrad <konrad@xenia.it>
+ *     0xfc17: Dave Konrad <konrad@xenia.it>
  *     0xfc1a: George Betzos <betzos@engr.colostate.edu>
+ *     0xfc1d: Arthur Liu <armie@slap.mine.nu>
+ *     0xfcd1: Mr. Dave Konrad <konrad@xenia.it>
  *
  * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  *
  *   This code is covered by the GNU GPL and you are free to make any
  *   changes you wish to it under the terms of the license. However the
  *   code has the potential to render your computer and/or someone else's
- *   unuseable. Unless you truely understand what is going on, I urge you
- *   not to make any modifications and use it as it stands.
+ *   unusable. Please proceed with care when modifying the code.
+ *
+ * Note: Unfortunately the laptop hardware can close the System Configuration
+ *       Interface on it's own accord. It is therefore necessary for *all*
+ *       programs using this driver to be aware that *any* SCI call can fail at
+ *       *any* time. It is up to any program to be aware of this eventuality
+ *       and take appropriate steps.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -37,7 +50,13 @@
  *
  */
 
-#define TOSH_VERSION "1.6 11/2/2000"
+/* this appears to be needed for gcc 2.9.5 and upwards */
+/*
+#define KERNEL_STRICT_NAMES
+*/
+
+
+#define TOSH_VERSION "1.8 4/2/2001"
 #define TOSH_DEBUG 0
 
 #ifdef MODULE
@@ -46,16 +65,17 @@
 #endif
 #include<linux/kernel.h>
 #include<linux/sched.h>
-/* this appears to be needed for gcc 2.9.5! */
-#ifdef __KERNEL_STRICT_NAMES
-#define _LOOSE_KERNEL_NAMES
-#endif
 #include<linux/types.h>
 #include<linux/fcntl.h>
 #include<linux/miscdevice.h>
 #include<linux/ioport.h>
 #include<asm/io.h>
+#if LINUX_VERSION_CODE > 0x020100
+#include<asm/uaccess.h>
 #include<asm/init.h>
+#else
+#include<asm/segment.h>
+#endif
 #ifdef CONFIG_PROC_FS
 #include<linux/stat.h>
 #include<linux/proc_fs.h>
@@ -70,8 +90,6 @@ static int tosh_bios = 0x0000;
 static int tosh_date = 0x0000;
 static int tosh_sci = 0x0000;
 static int tosh_fan = 0;
-static int tosh_start = 0;
-static int tosh_extent = 0;
 
 int tosh_fn = 0;
 
@@ -283,7 +301,7 @@ static int tosh_smm(SMMRegisters *regs)
 		"andl $1,%%eax\n" \
 		: "=a" (eax)
 		: "a" (regs)
-		: "%ebx", "%ecx", "%edx", "%esi", "%edi");
+		: "%ebx", "%ecx", "%edx", "%esi", "%edi", "memory");
 
 	return eax;
 }
@@ -308,18 +326,23 @@ static int tosh_release(struct inode *ip, struct file *fp)
 static int tosh_ioctl(struct inode *ip, struct file *fp, unsigned int cmd,
 	unsigned long arg)
 {
-	SMMRegisters *regs;
+	SMMRegisters regs;
 	unsigned short ax,bx;
 	int err;
 
 	if (!arg)
 		return -EINVAL;
 
-	regs = (SMMRegisters *) arg;
+#if LINUX_VERSION_CODE > 0x020200
+	copy_from_user(&regs, (SMMRegisters *) arg, sizeof(SMMRegisters));
+#else
+	memcpy_fromfs(&regs, (SMMRegisters *) arg, sizeof(SMMRegisters));
+#endif
+
 	switch (cmd) {
 		case TOSH_SMM:
-			ax = regs->eax & 0xff00;
-			bx = regs->ebx & 0xffff;
+			ax = regs.eax & 0xff00;
+			bx = regs.ebx & 0xffff;
 			/* block HCI calls to read/write memory & PCI devices */
 			if (((ax==0xff00) || (ax==0xfe00)) && (bx>0x0069))
 				return -EINVAL;
@@ -327,16 +350,22 @@ static int tosh_ioctl(struct inode *ip, struct file *fp, unsigned int cmd,
 			/* do we need to emulate the fan ? */
 			if (tosh_fan==1) {
 				if (((ax==0xf300) || (ax==0xf400)) && (bx==0x0004)) {
-					err = tosh_emulate_fan(regs);
+					err = tosh_emulate_fan(&regs);
 					break;
 				}
 			}
-			err = tosh_smm(regs);
+			err = tosh_smm(&regs);
 			break;
 		default:
 			return -EINVAL;
 	}
 
+#if LINUX_VERSION_CODE > 0x020200
+        copy_to_user((SMMRegisters *) arg, &regs, sizeof(SMMRegisters));
+#else
+        memcpy_tofs((SMMRegisters *) arg, &regs, sizeof(SMMRegisters));
+#endif
+
 	return (err==0) ? 0:-EINVAL;
 }
 
@@ -359,6 +388,7 @@ int tosh_get_info(char *buffer, char **start, off_t fpos, int length, int dummy)
 	     2) SCI version
 	     3) BIOS version (major, minor)
 	     4) BIOS date (in SCI date format)
+	     5) Fn Key status
 	*/
 
 	temp += sprintf(temp, "1.1 0x%04x %d.%d %d.%d 0x%04x 0x%02x\n",
@@ -381,10 +411,12 @@ int tosh_get_info(char *buffer, char **start, off_t fpos, int length, int dummy)
 static void tosh_set_fn_port(void)
 {
 	switch (tosh_id) {
-		case 0xfc1a: case 0xfc15:
+		case 0xfc04: case 0xfc09: case 0xfc0a: case 0xfc10: case 0xfc11:
+		case 0xfc13: case 0xfc15: case 0xfc1a:
 			tosh_fn = 0x62;
 			break;
-		case 0xfce0: case 0xfcd1: case 0xfce2: case 0xfc17:
+		case 0xfc08: case 0xfc17: case 0xfc1d: case 0xfcd1: case 0xfce0:
+		case 0xfce2:
 			tosh_fn = 0x68;
 			break;
 		default:
@@ -402,7 +434,7 @@ static void tosh_set_fn_port(void)
 static int tosh_get_machine_id(void)
 {
 	int id;
-	unsigned char ah;
+	SMMRegisters regs;
 	unsigned short bx,cx;
 	unsigned long address;
 
@@ -414,12 +446,11 @@ static int tosh_get_machine_id(void)
 
 		/* start by getting a pointer into the BIOS */
 
-		asm ("movw $0xc000,%%ax\n\t" \
-			"movw $0x0000,%%bx\n\t" \
-			"movw $0x0000,%%cx\n\t" \
-			"inb $0xb2,%%al\n\t" \
-			"movb %%ah,%%al\n" \
-			: "=b" (bx), "=a" (ah) : : "%ecx");
+		regs.eax = 0xc000;
+		regs.ebx = 0x0000;
+		regs.ecx = 0x0000;
+		tosh_smm(&regs);
+		bx = (unsigned short) (regs.ebx & 0xffff);
 
 		/* At this point in the Toshiba routines under MS Windows
 		   the bx register holds 0xe6f5. However my code is producing
@@ -427,7 +458,7 @@ static int tosh_get_machine_id(void)
 		   value. This has been verified on a Satellite Pro 430CDT,
 		   Tecra 750CDT, Tecra 780DVD and Satellite 310CDT. */
 #if TOSH_DEBUG
-		printk("toshiba: debugging ID bx=0x%04x\n", bx);
+		printk("toshiba: debugging ID ebx=0x%04x\n", regs.ebx);
 #endif
 		bx = 0xe6f5;
 
@@ -458,33 +489,26 @@ static int tosh_get_machine_id(void)
  */
 int tosh_probe(void)
 {
-	int major,minor,day,year,month;
-	unsigned short dx;
-	unsigned char ah,flags;
+	int major,minor,day,year,month,flag;
+	SMMRegisters regs;
 
 	/* call the Toshiba SCI support check routine */
-
-	asm ("movw $0xf0f0,%%ax\n\t" \
-		"movw $0x0000,%%bx\n\t" \
-		"movw $0x0000,%%cx\n\t" \
-		"inb $0xb2,%%al\n\t" \
-		"movb %%ah,%%bl\n\t" \
-		"lahf\n\t" \
-		"movb %%ah,%%al\n" \
-		: "=b" (ah), "=a" (flags), "=d" (dx)
-		:
-		: "%ecx");
+	
+	regs.eax = 0xf0f0;
+	regs.ebx = 0x0000;
+	regs.ecx = 0x0000;
+	flag = tosh_smm(&regs);
 
 	/* if this is not a Toshiba laptop carry flag is set and ah=0x86 */
 
-	if ((ah==0x86) || ((flags & 0x01)==1)) {
+	if ((flag==1) || ((regs.eax & 0xff00)==0x8600)) {
 		printk("toshiba: not a supported Toshiba laptop\n");
 		return -ENODEV;
 	}
 
 	/* if we get this far then we are running on a Toshiba (probably)! */
 
-	tosh_sci = (int) dx;
+	tosh_sci = regs.edx & 0xffff;
 	
 	/* next get the machine ID of the current laptop */
 
@@ -504,26 +528,13 @@ int tosh_probe(void)
 	tosh_date = (((year-90) & 0x1f)<<10) | ((month & 0xf)<<6)
 		| ((day & 0x1f)<<1);
 
-	/* what ports do we need to grab on this model for the fn key */
 
-	switch (tosh_id) {
-		case 0xfc1a:
-			tosh_start = 0x62;
-			tosh_extent = 1;
-		case 0xfce0: case 0xfcd1: case 0xfce2:
-			tosh_start = 0x68;
-			tosh_extent = 1;
-			break;
-		default: /* also used for fan on Portage 610CT and Tecra 700x */
-			tosh_start = 0xe4;
-			tosh_extent = 2;
-	}
-
-	/* we can't check the ports as the keyboard driver grabs 0x60-0x6f
-	   and the pic driver grabs 0xa0-0xbf. We just have to live
-	   dangerously and use the ports anyway */
-
-	request_region(tosh_start, tosh_extent, "toshiba");
+	/* in theory we should check the ports we are going to use for the
+	   fn key detection (and the fan on the Portage 610/Tecra700), and
+	   then request them to stop other drivers using them. However as
+	   the keyboard driver grabs 0x60-0x6f and the pic driver grabs
+	   0xa0-0xbf we can't. We just have to live dangerously and use the
+	   ports anyway, oh boy! */
 
 	/* do we need to emulate the fan? */
 
@@ -533,8 +544,11 @@ int tosh_probe(void)
 	return 0;
 }
 
-
+#if LINUX_VERSION_CODE > 0x020200
 __initfunc(int tosh_init(void))
+#else
+int tosh_init(void)
+#endif
 {
 	/* are we running on a Toshiba laptop */
 
@@ -584,10 +598,6 @@ void cleanup_module(void)
 
 	misc_deregister(&tosh_device);
 
-	/* release ports */
-
-	release_region(tosh_start, tosh_extent);
-
 	return;
 }
 #endif
