@@ -1,6 +1,6 @@
 /* wmtuxtime.c -- WindowMaker dockable applet portion of TuxTime
  *
- * Copyright (c) 1997-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1997-2001  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * Based on
  *       wmapm by Chris D. Faulhaber <jedgar@speck.ml.org> and
@@ -8,6 +8,18 @@
  *       mount.app by Steve Borho <borho@stlnet.com>
  *
  * $Log: wmtuxtime.c,v $
+ * Revision 1.8  2001/02/04 16:39:47  jab
+ * added a pile more machine ID's
+ * added notice about kernel module to error message
+ * moved the lock file to /var/tmp
+ * don't display battery warning messages when on mains power
+ * added -c option for higher contrast for DSTN screens
+ *
+ * Revision 1.7  2000/06/23 21:47:25  jab
+ * moved all the calls to determine power status into a single routine
+ * various code changes to make it more Libretto friendly
+ * added a good few machine ID's
+ *
  * Revision 1.6  2000/02/12 21:33:31  jab
  * added patch from Stephen Morphet <smorphet@iee.org> to use a fallback to
  * /proc/apm for time remaining when SCI_BATTERY_TIME does not work
@@ -46,7 +58,7 @@
  *
  */
 
-static char const rcsid[]="$Id: wmtuxtime.c,v 1.6 2000/02/12 21:33:31 jab Exp jab $";
+static char const rcsid[]="$Id: wmtuxtime.c,v 1.8 2001/02/04 16:39:47 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -78,7 +90,6 @@ static char const rcsid[]="$Id: wmtuxtime.c,v 1.6 2000/02/12 21:33:31 jab Exp ja
 
 
 enum { EMPTY, PRIMARY, AUXILLARY };
-enum { APM_READ, APM_CLEAR };
 
 static int id;
 static int power = SCI_MAINS;
@@ -88,17 +99,6 @@ static int alarms;
 static char *config;
 static BatteryState low,critical;
 
-/* Structure containing data read from /proc/apm */
-static struct apm_data_t
-{
-	/* Status */
-	int valid;
-	int read_failed;
-	/* Data */
-	int battery;
-	int time;
-} apm_data;
-
 
 /*
  * Catch any signals and exit the program nicely
@@ -108,7 +108,7 @@ void CatchSignal(int x)
 	fprintf(stderr, "wmtuxtime: caught signal %d, cleaning up...\n", x);
 
 	SciCloseInterface();
-	unlink(_PATH_VARRUN PID_FILE);
+	unlink(PID_FILE);
 	exit(0);
 }
 
@@ -147,14 +147,14 @@ void ProcessComandLine(int *argc, char ***argv)
 
 	for (i=1;i<*argc;i++) {
 		if ((!strcmp((*argv)[i], "-c")) || (!strcmp((*argv)[i], "--contrast"))) {
-/*			for (j=5;j<12;j++)
+			for (j=5;j<12;j++)
 				wmtuxtime_xpm[3][j] = '0';
 			for (j=5;j<12;j++)
 				wmtuxtime_xpm[4][j] = '0';
 			for (j=5;j<12;j++)
 				wmtuxtime_xpm[7][j] = '0';
 			for (j=5;j<12;j++)
-				wmtuxtime_xpm[9][j] = '0';*/
+				wmtuxtime_xpm[9][j] = '0';
 		}
 	}
 
@@ -462,8 +462,9 @@ int BatteryTest(BatteryState *settings, int percent, int time)
  */
 void PowerChanged(int source, char *file)
 {
-	unsigned short save,hdd,display,sleep;
-	unsigned short speed,lcd,autooff,cooling;
+	unsigned short save,hdd,display,sleep,speed;
+	unsigned short lcd,autooff,cooling,standby;
+	unsigned short lcdtype;
 	SMMRegisters reg;
 	int user;
 	char *section;
@@ -485,6 +486,7 @@ void PowerChanged(int source, char *file)
 	lcd = GetConfigInt(section, "LCDBrightness", 0, file);
 	autooff = GetConfigInt(section, "SystemAutoOff", 1, file);
 	cooling = GetConfigInt(section, "CoolingMethod", 0, file);
+	standby = GetConfigInt(section, "StandbyTime", 0, file);
 
 	SciOpenInterface();
 
@@ -498,6 +500,15 @@ void PowerChanged(int source, char *file)
 		user = 1;
 	}
 
+	/* determine whether to make LCD backlight or brightness call */
+
+	reg.ebx = SCI_LCD_BRIGHTNESS;
+	if (SciGet(&reg)==SCI_SUCCESS) {
+		lcdtype = SCI_LCD_BRIGHTNESS;
+	} else {
+		lcdtype = SCI_LCD_BACKLIGHT;
+	}
+
 	/* set the other settings if user mode or new style machine */
 
 	if ((save==SCI_USER_SETTINGS) || (user==1)) {
@@ -525,7 +536,7 @@ void PowerChanged(int source, char *file)
 			fprintf(stderr, "wmtuxtime: unable to set CPU "
 				"speed\n");
 		}
-		reg.ebx = SCI_LCD_BRIGHTNESS;
+		reg.ebx = lcdtype;
 		reg.ecx = lcd;
 		if (SciSet(&reg)==SCI_FAILURE) {
 			fprintf(stderr, "wmtuxtime: unable to set LCD "
@@ -546,18 +557,20 @@ void PowerChanged(int source, char *file)
 
 		/* toggle battery save mode to ensure settings take effect */
 
-		reg.ebx = SCI_BATTERY_SAVE;
-		reg.ecx = 1;
-		if (SciSet(&reg)==SCI_FAILURE) {
-			fprintf(stderr, "wmtuxtime: unable to set battery "
-				"save mode (1)\n");
-		}
+		if (user==0) {
+			reg.ebx = SCI_BATTERY_SAVE;
+			reg.ecx = 1;
+			if (SciSet(&reg)==SCI_FAILURE) {
+				fprintf(stderr, "wmtuxtime: unable to set "
+					"battery save mode (1)\n");
+			}
 
-		reg.ebx = SCI_BATTERY_SAVE;
-		reg.ecx = save;
-		if (SciSet(&reg)==SCI_FAILURE) {
-			fprintf(stderr, "wmtuxtime: unable to set battery "
-				"save mode (save)\n");
+			reg.ebx = SCI_BATTERY_SAVE;
+			reg.ecx = save;
+			if (SciSet(&reg)==SCI_FAILURE) {
+				fprintf(stderr, "wmtuxtime: unable to set "
+					"battery save mode (save)\n");
+			}
 		}
 	}
 
@@ -573,9 +586,18 @@ void PowerChanged(int source, char *file)
  */
 void DisplayAlarm(int state, int percent, int time)
 {
-	int trigger,hour,minute;
+	int i,trigger,hour,minute;
  	char message[128];
 
+
+	/* Don't bother with the alarms if we are on mains power */
+
+	i = SciACPower();
+	if ((i==SCI_MAINS) && (i!=0))
+		return;
+
+	/* send out the emergency alarm first */
+
 	if (state==EMERGENCY) {
 		XBell(display, 100);
 		sprintf(message, "%s", EMERGENCY_ALERT);
@@ -616,92 +638,89 @@ void DisplayAlarm(int state, int percent, int time)
 
 
 /*
- * Get information from /proc/apm if avaliable.
+ * Get the power status i.e. percent, time remaining, battery charging etc.
  */
-void ReadApm(int clear)
+int PowerStatus(int *charging, int *time, int *percent, int *second)
 {
 	FILE *str;
+	SMMRegisters reg;
+	int hours,minutes;
 	char version[10],buffer[64];
 
-	if (clear==APM_CLEAR) {
-		/* invalidate the stored data */
-		apm_data.valid = 0;
-		return;
-	}
 
-	/* If /proc/apm read has failed in the past, let's assume it
-	   will fail again.  Otherwise, only read if it has not been
-	   read since the last clear. */
+	if (access("/proc/apm", R_OK)) {
+		/* cannot find /proc/apm */
+		fprintf(stderr, "wmtuxtime: unable to access /proc/apm\n");
+		return 0;
+	} else {
 
-	if((!apm_data.read_failed) && (!apm_data.valid)) {
-		if (access("/proc/apm", R_OK)) {
-			/* cannot find /proc/apm */
-			fprintf(stderr, 
-				"wmtuxtime: unable to access /proc/apm\n");
-			apm_data.read_failed = 1;
-			return;
-		} else {
-			/* open /proc/apm for reading */
-			if (!(str = fopen("/proc/apm", "r"))) {
-				fprintf(stderr, "wmtuxtime: "
-					"unable to open /proc/apm\n");
-				apm_data.read_failed = 1;
-				return;
-			}
+		/* open /proc/apm for reading */
+		if (!(str = fopen("/proc/apm", "r"))) {
+			fprintf(stderr, "wmtuxtime: unable to open /proc/apm\n");
+			return 0;
+		}
 
-			/* scan in the information */
-			fgets(buffer, sizeof(buffer)-1, str);
-			fclose(str);
-			buffer[sizeof(buffer)-1] = '\0';
+		/* scan in the information */
+		fgets(buffer, sizeof(buffer)-1, str);
+		fclose(str);
+		buffer[sizeof(buffer)-1] = '\0';
+		sscanf(buffer, "%s %*d.%*d %*x %*x %x %*x %d%% %d %*s\n",
+			version, charging, percent, time);
 
-			sscanf(buffer, 
-				"%s %*d.%*d %*x %*x %x %*x %*d%% %d %*s\n",
-				version, &(apm_data.battery), &(apm_data.time));
-	  
-			/* if old style driver return */
-			if (version[0]=='B') {
-				fprintf(stderr, 
-					"wmtuxtime: old style apm driver\n");
-				apm_data.read_failed = 1;
-				return;
-			}
-
-			apm_data.valid = 1;
+		/* if old style driver return */
+		if (version[0]=='B') {
+			fprintf(stderr, "wmtuxtime: old style apm driver\n");
+			return 0;
 		}
 	}
 
-	return;
-}
+	/* get the percent and time remaining using SCI method */
 
+	if (SciOpenInterface()==SCI_SUCCESS) {
+		reg.ebx = SCI_BATTERY_PERCENT;
+		if (SciGet(&reg)==SCI_SUCCESS)
+			*percent = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
+		else
+			*percent = -1;
 
-/*
- * Determine if the battery is charging by reading /proc/apm if avaliable
- */
-int BatteryCharging(void)
-{
-	ReadApm( APM_READ );
-	return apm_data.battery;
-}
+		/* some times percent fails so catch to prevent display flickering */
 
+		if ((*percent==-1) && (oldpercent!=-1)) {
+			*percent = oldpercent;
+			oldpercent = -1;
+		} else {
+			oldpercent = *percent;
+		}
 
-/*
- * Fallback routine for when SCI_BATTERY_TIME fails to use /proc/apm
- */
-int TimeRemaining(void)
-{
-	int hours = 0;
-	int minutes;
+		reg.ebx = SCI_BATTERY_TIME;
+		if (SciGet(&reg)==SCI_SUCCESS) {
+			*time = (reg.ecx & 0xffff);
+		} else {
+			/* fall back to using the /proc/apm time data */
+			hours = *time/60;
+			minutes = *time-(60*hours);
+			*time = SCI_TIME(hours, minutes);
+		}
 
-	ReadApm(APM_READ);
+		/* check to see if their is a battery in the SelectBay */
 
-	if(apm_data.valid) {
-		minutes = apm_data.time;
-		hours = minutes/60;
-		minutes -= (60*hours);
-		return SCI_TIME(hours,minutes);
-        }
+		reg.ebx = SCI_2ND_BATTERY;
+		if (SciGet(&reg)==SCI_SUCCESS) {
+			if ((reg.ecx & 0xffff)!=0xffff)
+				*second = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
+			else
+				*second = -1;
+		} else {
+			selectbay = EMPTY;
+			*second = -1;
+		}
 
-	return -1;
+		SciCloseInterface();
+	} else {
+		return 0;
+	}
+
+	return 1;
 }
 
 
@@ -711,54 +730,18 @@ int TimeRemaining(void)
  */
 void Update(void)
 {
-	SMMRegisters reg;
-	int time,percent,second,battery;
-	int hour,minute,state,i,j;
-	int time_is_apm;
+	int time,percent,second,charging;
+	int hour,minute,state,battery,i;
 
-	time = 0;
-	time_is_apm = 0;
-	percent = 0;
 
-	/* clear the apm data, it will be read again next time it's needed. */
-	ReadApm(APM_CLEAR);
+	/* get the power status of the laptop */
 
-	/* get the percent and time remaining */
+	if (PowerStatus(&charging, &time, &percent, &second)==0)
+		return;
 
-	SciOpenInterface();
-	reg.ebx = SCI_BATTERY_PERCENT;
-	if (SciGet(&reg)==SCI_SUCCESS)
-		percent = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
-	else
-		percent = -1;
+	/* activate button if battery in the SelectBay */
 
-	/* some times percent fails so catch to prevent display flickering */
-
-	if ((percent==-1) && (oldpercent!=-1)) {
-		percent = oldpercent;
-		oldpercent = -1;
-	} else {
-		oldpercent = percent;
-	}
-
-	reg.ebx = SCI_BATTERY_TIME;
-	if (SciGet(&reg)==SCI_SUCCESS) {
-		time = (reg.ecx & 0xffff);
-	} else {
-		/* fall back to reading /proc/apm. */
-
-		time = TimeRemaining();
-		time_is_apm = 1;
-	}
-
-	/* check to see if their is a battery in the SelectBay */
-
-	reg.ebx = SCI_2ND_BATTERY;
-	if (SciGet(&reg)==SCI_SUCCESS) {
-		if ((reg.ecx & 0xffff)!=0xffff)
-			second = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
-		else
-			second = -1;
+	if (second>=0) {
 		if ((power==SCI_BATTERY) && (selectbay==EMPTY))
 			copyXPMArea(62, 93, 15, 10, 44, 33);
 		if (selectbay==EMPTY) {
@@ -770,26 +753,26 @@ void Update(void)
 		}
 	} else {
 		selectbay = EMPTY;
-		second = -1;
 		/* grey out the arrow button */
 		copyXPMArea(119, 35, 4, 7,  11, 50);
 		copyXPMArea(119, 35, 4, 7,  71, 58);
 		copyXPMArea(119, 35, 4, 7,  71, 46);
 	}
 
-	SciCloseInterface();
-
 	/* calculate the battery condition */
 
 	state = NORMAL;
 	hour = SCI_HOUR(time);
 	minute = SCI_MINUTE(time);
-	if ((percent<3) && (percent>=0))
-		state = EMERGENCY;
-	else if (BatteryTest(&critical, percent, MINUTES(hour, minute))!=0)
-		state = CRITICAL;
-	else if (BatteryTest(&low, percent, MINUTES(hour, minute))!=0)
-		state = LOW;
+	
+	if ((percent>=0) && (time>=0)) {
+		if ((percent<3) && (percent>=0))
+			state = EMERGENCY;
+		else if (BatteryTest(&critical, percent, MINUTES(hour, minute))!=0)
+			state = CRITICAL;
+		else if (BatteryTest(&low, percent, MINUTES(hour, minute))!=0)
+			state = LOW;
+	}
 
 	/* send out the battery alarms */
 
@@ -844,7 +827,7 @@ void Update(void)
 
 	/* check to see if the batteries are charging */
 
-	if (BatteryCharging()==3) {
+	if (charging==3) {
 		copyXPMArea(119, 5, 5, 7,  6, 7);
 		copyXPMArea(83, 69, 5, 9, 15, 7);
 	}
@@ -853,7 +836,7 @@ void Update(void)
 	     If not charging: Time left before battery drains to 0% 
 	     If charging:     Time left before battery gets to maximum */
 
-	if (time<0 || ((time_is_apm) && (power==SCI_MAINS))) {
+	if (time<0) {
 		/* dodgy time so just display a "null" indicator (--:--) */
 
 		copyXPMArea(83, 69, 41, 9, 15, 7);
@@ -898,22 +881,21 @@ void Update(void)
 
 		/* Show Meter */
 
-		i = battery*49/100;
 		switch (state) {
 			case NORMAL:
-				j = 14;
+				i = 14;
 				break;
 			case LOW:
-				j = 24;
+				i = 24;
 				break;
 			case CRITICAL:
-				j = 34;
+				i = 34;
 				break;
 			default:
-				j = 14;
+				i = 14;
 		}
 
-		copyXPMArea(66, j, i, 9, 7, 21);
+		copyXPMArea(66, i, (battery*49)/100, 9, 7, 21);
 	}
 		
 	return;
@@ -925,15 +907,16 @@ void Update(void)
  */
 void HandleDeadChild(pid_t pid, int status)
 {
-	if (WIFEXITED(status) && WEXITSTATUS(status)==0x10) {
-		/* reread the alarm settings */
-		SetupAlarms(config);
-		Update();
-	}
-
-	if (WIFEXITED(status) && WEXITSTATUS(status)==0x20) {
-		/* set the battery save options */
-		PowerChanged(power, config);
+	if (WIFEXITED(status)) {
+		if (WEXITSTATUS(status)==0x10) {
+			/* reread the alarm settings */
+			SetupAlarms(config);
+			Update();
+		}	
+		if (WEXITSTATUS(status)==0x20) {
+			/* set the battery save options */
+			PowerChanged(power, config);
+		}
 	}
 
 	return;
@@ -941,8 +924,7 @@ void HandleDeadChild(pid_t pid, int status)
 
 
 /*
- * This function is called from the main loop. It's job is to find
- * dead children.
+ * Called from the main loop, it's job is to find dead children.
  */
 void ShovelDeadChildren()
 {
@@ -973,7 +955,8 @@ int main(int argc, char *argv[])
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
-		fprintf(stderr, "wmtuxtime: this computer is not supported\n");
+		fprintf(stderr, "wmtuxtime: this computer is not supported "
+			"or the kernel module not installed.\n");
 		return 1;
 	}
 
@@ -995,7 +978,14 @@ int main(int argc, char *argv[])
         setegid(getgid());
 
 	switch (id) {
+		case 0xfc00: /* Satellite 2140CDS/2180CDT */
+		case 0xfc01: /* Satellite 2710xDVD */
+		case 0xfc02: /* Satellite Pro 4270CDT/4300CDT/4340CDT */
+		case 0xfc04: /* Portege 3410CT, 3440CT */
+		case 0xfc08: /* Satellite 2100CDS/CDT 1550CDS */
 		case 0xfc09: /* Satellite 2610CDT, 2650XDVD */
+		case 0xfc0a: /* Portage 7140 */
+		case 0xfc0b: /* Satellite Pro 4200 */
 		case 0xfc0c: /* Tecra 8100x */
 		case 0xfc0f: /* Satellite 2060CDS/CDT */
 		case 0xfc10: /* Satellite 2550/2590 */
@@ -1014,6 +1004,9 @@ int main(int argc, char *argv[])
 		case 0xfc40: /* T4500C */
 		case 0xfc41: /* T4500 */
 		case 0xfc45: /* T4400SX/SXC */
+		case 0xfc51: /* Satellite 2210CDT, 2770XDVD */
+		case 0xfc52: /* Dynabook Satellite DB60P/4DA */
+		case 0xfc54: /* Satellite 2800DVD */
 		case 0xfc5f: /* T3300SL */
 		case 0xfc69: /* T1900C */
 		case 0xfc6a: /* T1900 */
@@ -1087,8 +1080,8 @@ int main(int argc, char *argv[])
 
 	/* check to see if a copy of wmTuxTime is already running */
 
-	if (!access(_PATH_VARRUN PID_FILE, R_OK)) {
-		if ((str = fopen(_PATH_VARRUN PID_FILE, "r" ))) {
+	if (!access(PID_FILE, R_OK)) {
+		if ((str = fopen(PID_FILE, "r" ))) {
 			fscanf(str, "%d", &pid);
 			fclose(str);
 
@@ -1102,14 +1095,14 @@ int main(int argc, char *argv[])
 
 			fprintf(stderr, "wmtuxtime: process %d appears to have "
 				"died, continuing\n", pid);
-			unlink(_PATH_VARRUN PID_FILE);
+			unlink(PID_FILE);
 		}
 	}
 
 	/* create the pid file */
 
 	pid = getpid();
-	if ((str = fopen(_PATH_VARRUN PID_FILE, "w"))) {
+	if ((str = fopen(PID_FILE, "w"))) {
 		fprintf(str, "%d\n", pid);
 		fclose(str);
 	}
